<?php

declare(strict_types=1);

namespace LaminasTest\Stdlib\TestAsset;

class ArrayObjectObjectVars
{
    /** @var string */
    public $public = 'prop:public';

    /** @var string */
    protected $protected = 'prop:protected';

    private string $private = 'prop:private';
}
