<?php

declare(strict_types=1);

namespace Laminas\Stdlib\Exception;

/**
 * Bad method call exception
 */
class BadMethodCallException extends \BadMethodCallException implements ExceptionInterface
{
}
